package com.mall.product.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mall.common.domain.vo.result.PageResult;
import com.mall.product.domain.dto.CategoryBrandRelationDto;
import com.mall.product.domain.entity.BrandEntity;
import com.mall.product.domain.entity.CategoryBrandRelationEntity;
import com.mall.product.domain.entity.CategoryEntity;
import com.mall.product.domain.vo.CategoryBrandRelationVo;
import com.mall.product.mapper.BrandMapper;
import com.mall.product.mapper.CategoryBrandRelationMapper;
import com.mall.product.mapper.CategoryMapper;
import com.mall.product.service.CategoryBrandRelationService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 品牌分类关联 服务实现类
 * </p>
 *
 * @author Bunny
 * @since 2025-07-04 14:05:53
 */
@Service
@Transactional
@RequiredArgsConstructor
public class CategoryBrandRelationServiceImpl extends ServiceImpl<CategoryBrandRelationMapper, CategoryBrandRelationEntity> implements CategoryBrandRelationService {

    private final CategoryMapper categoryMapper;
    private final BrandMapper brandMapper;

    /**
     * * 品牌分类关联 服务实现类
     *
     * @param pageParams 品牌分类关联分页查询page对象
     * @param dto        品牌分类关联分页查询对象
     * @return 查询分页品牌分类关联返回对象
     */
    @Override
    public PageResult<CategoryBrandRelationVo> getCategoryBrandRelationPage(Page<CategoryBrandRelationEntity> pageParams, CategoryBrandRelationDto dto) {
        IPage<CategoryBrandRelationVo> page = baseMapper.selectListByPage(pageParams, dto);

        return PageResult.<CategoryBrandRelationVo>builder()
                .list(page.getRecords())
                .pageNo(page.getCurrent())
                .pageSize(page.getSize())
                .total(page.getTotal())
                .build();
    }

    /**
     * 添加品牌与分类的关联关系
     *
     * <p>实现逻辑：
     * 1. 根据品牌ID查询品牌信息，获取品牌名称
     * 2. 根据分类ID查询分类信息，获取分类名称
     * 3. 将品牌和分类的关联关系及冗余信息保存到关联表中
     *
     * <p>设计说明：
     * 采用冗余存储设计，在关联表中同时存储品牌名称和分类名称，
     * 避免多表关联查询，提升电商系统在大数据量场景下的查询性能
     *
     * @param dto 品牌分类关联数据 {@link CategoryBrandRelationDto}
     */
    @Override
    public void saveCategoryBrandRelation(CategoryBrandRelationDto dto) {
        // 根据品牌id查询品牌信息
        Long brandId = dto.getBrandId();
        BrandEntity brandEntity = brandMapper.selectById(brandId);

        // 根据分类id查询分类信息
        Long catelogId = dto.getCatelogId();
        CategoryEntity categoryEntity = categoryMapper.selectById(catelogId);

        // 将查询到的
        CategoryBrandRelationEntity categoryBrandRelationEntity = new CategoryBrandRelationEntity();
        BeanUtils.copyProperties(dto, categoryBrandRelationEntity);
        categoryBrandRelationEntity.setBrandName(brandEntity.getName());
        categoryBrandRelationEntity.setCatelogName(categoryEntity.getName());
        save(categoryBrandRelationEntity);
    }

    /**
     * 更新品牌分类关联
     *
     * @param dto 品牌分类关联更新
     */
    @Override
    public void updateCategoryBrandRelation(CategoryBrandRelationDto dto) {
        CategoryBrandRelationEntity categoryBrandRelationEntity = new CategoryBrandRelationEntity();
        BeanUtils.copyProperties(dto, categoryBrandRelationEntity);
        updateById(categoryBrandRelationEntity);
    }

    /**
     * 删除|批量删除品牌分类关联
     *
     * @param ids 删除id列表
     */
    @Override
    public void deleteCategoryBrandRelation(List<Long> ids) {
        removeByIds(ids);
    }

    /**
     * 返回分类列表
     *
     * @param brandId 品牌id
     * @return 品牌关联所有分类列表
     */
    @Override
    public List<CategoryBrandRelationVo> getCategoryList(Long brandId) {
        // 构建查询条件
        QueryWrapper<CategoryBrandRelationEntity> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(CategoryBrandRelationEntity::getBrandId, brandId);

        // 查询品牌分类关联的实体类对象
        List<CategoryBrandRelationEntity> list = list(wrapper);
        return list.stream().map(categoryBrandRelationEntity -> {
                    CategoryBrandRelationVo vo = new CategoryBrandRelationVo();
                    BeanUtils.copyProperties(categoryBrandRelationEntity, vo);
                    return vo;
                })
                .toList();
    }

}